==> objectify-alpha_28.4.tgz: objectify-alpha_28.4/url.c <==
/*
--             This file is part of the New World OS project
--                   Copyright (C) 2007  QRW Software
--           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--                      http://www.qrwsoftware.com
--                      http://nwos.sourceforge.com
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--
-- $Log: url.c,v $
-- Revision 1.2  2007/07/01 19:44:12  jsedwards
-- Upgrade to GPLv3.
--
-- Revision 1.1  2007/04/22 14:25:45  jsedwards
-- Initial version.
--
*/


#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>   /* define memset */

#include "crc32.h"
#include "objectify_private.h"



static size_t get_url_object_size(void* url_obj)
{
    assert(((C_struct_Uniform_Resource_Locator*)url_obj)->count > 0);

    return sizeof(C_struct_Uniform_Resource_Locator) + ((C_struct_Uniform_Resource_Locator*)url_obj)->count;
}


bool nwos_find_public_url(char* url, ObjRef* ref)
{
    C_struct_Class_Definition class_def_obj;
    uint8 kludge[FILE_BLOCK_SIZE];
    C_struct_Uniform_Resource_Locator* ptr_url_obj = (C_struct_Uniform_Resource_Locator*)kludge;
    ObjRef public_url_class_ref;
    ObjRef object_class;
    ReferenceList* ref_list;
    int num_refs;
    size_t length;
    int i;


    length = strlen(url);

    assert(length <= 255);
    
    nwos_find_public_class_definition("UNIFORM RESOURCE LOCATOR", &public_url_class_ref);

    nwos_read_class_definition(&public_url_class_ref, &class_def_obj);

    ref_list = nwos_malloc_reference_list(&class_def_obj.header.object.references);

    num_refs = ref_list->common_header.num_refs;

    /* printf("num_refs: %d\n", num_refs); */

    for (i = 0; i < num_refs; i++)
    {
	nwos_get_object_class(&ref_list->references[i], &object_class);

	if (is_same_object(&object_class, &public_url_class_ref))
	{
	    nwos_read_variable_sized_object_from_disk(&ref_list->references[i], kludge, sizeof(kludge), &get_url_object_size);

	    /* remember ptr_url_obj points to the kludge buffer */

	    if (ptr_url_obj->count == length && strncasecmp((char*)ptr_url_obj->storage, url, length) == 0)   /* found a match */
	    {
		memcpy(ref, &ref_list->references[i], sizeof(ObjRef));
		break;
	    }
	}
    }

    nwos_free_reference_list(ref_list);
    ref_list = NULL;

    return (i != num_refs);   /* return true if we found it */
}



/* Find existing url or create new */

#ifdef PUBLIC_MODE
ObjCreateResult nwos_create_url(char* url, ObjRef* ref)
{
    uint8 kludge[FILE_BLOCK_SIZE];
    C_struct_Uniform_Resource_Locator* ptr_url_obj = (C_struct_Uniform_Resource_Locator*)kludge;
    size_t length;
    ObjRef public_url_class_ref;
    ObjCreateResult result = FOUND_EXISTING;
    int i;
    C_struct_Class_Definition class_def_obj;

    length = strlen(url);

    assert(length <= 255);

    /* first find out if we already have this url */

    if (!nwos_find_public_url(url, ref))   /* didn't find it */
    {
	nwos_find_public_class_definition("UNIFORM RESOURCE LOCATOR", &public_url_class_ref);

	memset(kludge, 0, sizeof(kludge));  /* zero it out */

	/* remember ptr_url_obj points to the kludge buffer */

	nwos_generate_new_id(ref);

	nwos_fill_in_common_header(&ptr_url_obj->header.common, ref, &public_url_class_ref);

	nwos_read_class_definition(&public_url_class_ref, &class_def_obj);

	ptr_url_obj->count = length;
	for (i = 0; i < length; i++) ptr_url_obj->storage[i] = tolower(url[i]);

	nwos_create_reference_list(ref, &ptr_url_obj->header.object.references);

	nwos_crc32_calculate((uint8*) &ptr_url_obj->header.object, sizeof(ObjectHeader), ptr_url_obj->header.common.header_chksum);

	nwos_crc32_calculate((uint8*) &ptr_url_obj->start_date, sizeof(C_struct_Uniform_Resource_Locator) + length - sizeof(EveryObject), ptr_url_obj->header.common.data_chksum);

	nwos_write_object_to_disk(ref, kludge, sizeof(C_struct_Uniform_Resource_Locator) + length);

	nwos_add_to_references(ref, &public_url_class_ref);

	ptr_url_obj = malloc(sizeof(C_struct_Uniform_Resource_Locator) + length);
	nwos_read_object_from_disk(ref, ptr_url_obj, sizeof(C_struct_Uniform_Resource_Locator) + length);
	assert(memcmp(kludge, ptr_url_obj, sizeof(C_struct_Uniform_Resource_Locator) + length) == 0);

	memset(kludge, 0, sizeof(kludge));  /* clear it */
	nwos_read_variable_sized_object_from_disk(ref, kludge, sizeof(kludge), &get_url_object_size);  /* read the other way */
	assert(memcmp(ptr_url_obj, kludge, sizeof(C_struct_Uniform_Resource_Locator) + length) == 0);

	free(ptr_url_obj);
	ptr_url_obj = NULL;

	result = CREATED_NEW;
    }

    return result;
}
#endif


bool nwos_url_to_string(ObjRef* ref, char* string, size_t size)
{
    uint8 kludge[FILE_BLOCK_SIZE];
    C_struct_Uniform_Resource_Locator* ptr_url_obj = (C_struct_Uniform_Resource_Locator*)kludge;


    nwos_read_variable_sized_object_from_disk(ref, kludge, sizeof(kludge), &get_url_object_size);

    /* remember ptr_url_obj points to the kludge buffer */

    if (size < (ptr_url_obj->count + 1))
    {
	return false;
    }

    memcpy(string, ptr_url_obj->storage, ptr_url_obj->count);
    string[ptr_url_obj->count] = '\0';

    return true;
}

==> objectify-alpha_29.5.tgz: objectify-alpha_29.5/url.c <==
/*
--             This file is part of the New World OS project
--                 Copyright (C) 2007-2008  QRW Software
--           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--                      http://www.qrwsoftware.com
--                      http://nwos.sourceforge.com
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--
-- $Log: url.c,v $
-- Revision 1.4  2008/09/01 00:10:55  jsedwards
-- Fix copyright year.  NO code changes.
--
-- Revision 1.3  2008/08/31 21:53:54  jsedwards
-- Added an assert around calls to nwos_read_variable_sized_object_from_disk
-- and nwos_read_object_from_disk because now they return false when they fail
-- instead of asserting themselves.
--
-- Revision 1.2  2007/07/01 19:44:12  jsedwards
-- Upgrade to GPLv3.
--
-- Revision 1.1  2007/04/22 14:25:45  jsedwards
-- Initial version.
--
*/


#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>   /* define memset */

#include "crc32.h"
#include "objectify_private.h"



static size_t get_url_object_size(void* url_obj)
{
    assert(((C_struct_Uniform_Resource_Locator*)url_obj)->count > 0);

    return sizeof(C_struct_Uniform_Resource_Locator) + ((C_struct_Uniform_Resource_Locator*)url_obj)->count;
}


bool nwos_find_public_url(char* url, ObjRef* ref)
{
    C_struct_Class_Definition class_def_obj;
    uint8 kludge[FILE_BLOCK_SIZE];
    C_struct_Uniform_Resource_Locator* ptr_url_obj = (C_struct_Uniform_Resource_Locator*)kludge;
    ObjRef public_url_class_ref;
    ObjRef object_class;
    ReferenceList* ref_list;
    int num_refs;
    size_t length;
    int i;


    length = strlen(url);

    assert(length <= 255);
    
    nwos_find_public_class_definition("UNIFORM RESOURCE LOCATOR", &public_url_class_ref);

    nwos_read_class_definition(&public_url_class_ref, &class_def_obj);

    ref_list = nwos_malloc_reference_list(&class_def_obj.header.object.references);

    num_refs = ref_list->common_header.num_refs;

    /* printf("num_refs: %d\n", num_refs); */

    for (i = 0; i < num_refs; i++)
    {
	nwos_get_object_class(&ref_list->references[i], &object_class);

	if (is_same_object(&object_class, &public_url_class_ref))
	{
	    assert(nwos_read_variable_sized_object_from_disk(&ref_list->references[i], kludge, sizeof(kludge), &get_url_object_size));

	    /* remember ptr_url_obj points to the kludge buffer */

	    if (ptr_url_obj->count == length && strncasecmp((char*)ptr_url_obj->storage, url, length) == 0)   /* found a match */
	    {
		memcpy(ref, &ref_list->references[i], sizeof(ObjRef));
		break;
	    }
	}
    }

    nwos_free_reference_list(ref_list);
    ref_list = NULL;

    return (i != num_refs);   /* return true if we found it */
}



/* Find existing url or create new */

#ifdef PUBLIC_MODE
ObjCreateResult nwos_create_url(char* url, ObjRef* ref)
{
    uint8 kludge[FILE_BLOCK_SIZE];
    C_struct_Uniform_Resource_Locator* ptr_url_obj = (C_struct_Uniform_Resource_Locator*)kludge;
    size_t length;
    ObjRef public_url_class_ref;
    ObjCreateResult result = FOUND_EXISTING;
    int i;
    C_struct_Class_Definition class_def_obj;

    length = strlen(url);

    assert(length <= 255);

    /* first find out if we already have this url */

    if (!nwos_find_public_url(url, ref))   /* didn't find it */
    {
	nwos_find_public_class_definition("UNIFORM RESOURCE LOCATOR", &public_url_class_ref);

	memset(kludge, 0, sizeof(kludge));  /* zero it out */

	/* remember ptr_url_obj points to the kludge buffer */

	nwos_generate_new_id(ref);

	nwos_fill_in_common_header(&ptr_url_obj->header.common, ref, &public_url_class_ref);

	nwos_read_class_definition(&public_url_class_ref, &class_def_obj);

	ptr_url_obj->count = length;
	for (i = 0; i < length; i++) ptr_url_obj->storage[i] = tolower(url[i]);

	nwos_create_reference_list(ref, &ptr_url_obj->header.object.references);

	nwos_crc32_calculate((uint8*) &ptr_url_obj->header.object, sizeof(ObjectHeader), ptr_url_obj->header.common.header_chksum);

	nwos_crc32_calculate((uint8*) &ptr_url_obj->start_date, sizeof(C_struct_Uniform_Resource_Locator) + length - sizeof(EveryObject), ptr_url_obj->header.common.data_chksum);

	nwos_write_object_to_disk(ref, kludge, sizeof(C_struct_Uniform_Resource_Locator) + length);

	nwos_add_to_references(ref, &public_url_class_ref);

	ptr_url_obj = malloc(sizeof(C_struct_Uniform_Resource_Locator) + length);
	assert(nwos_read_object_from_disk(ref, ptr_url_obj, sizeof(C_struct_Uniform_Resource_Locator) + length));
	assert(memcmp(kludge, ptr_url_obj, sizeof(C_struct_Uniform_Resource_Locator) + length) == 0);

	memset(kludge, 0, sizeof(kludge));  /* clear it */
	assert(nwos_read_variable_sized_object_from_disk(ref, kludge, sizeof(kludge), &get_url_object_size));  /* read the other way */
	assert(memcmp(ptr_url_obj, kludge, sizeof(C_struct_Uniform_Resource_Locator) + length) == 0);

	free(ptr_url_obj);
	ptr_url_obj = NULL;

	result = CREATED_NEW;
    }

    return result;
}
#endif


bool nwos_url_to_string(ObjRef* ref, char* string, size_t size)
{
    uint8 kludge[FILE_BLOCK_SIZE];
    C_struct_Uniform_Resource_Locator* ptr_url_obj = (C_struct_Uniform_Resource_Locator*)kludge;


    assert(nwos_read_variable_sized_object_from_disk(ref, kludge, sizeof(kludge), &get_url_object_size));

    /* remember ptr_url_obj points to the kludge buffer */

    if (size < (ptr_url_obj->count + 1))
    {
	return false;
    }

    memcpy(string, ptr_url_obj->storage, ptr_url_obj->count);
    string[ptr_url_obj->count] = '\0';

    return true;
}
